"""Constants for the UniFi component."""
import logging

LOGGER = logging.getLogger('.')
DOMAIN = 'unifi'

CONTROLLER_ID = '{host}-{site}'

CONF_CONTROLLER = 'controller'
CONF_POE_CONTROL = 'poe_control'
CONF_SITE_ID = 'site'
